import unittest
from dot.dicy import dicy
import pickle

class test_dicy(unittest.TestCase):
    def test(self):
        d = dicy()
        with self.assertRaises(AttributeError):
            d.a
            
        d["a"] = 1
        self.assertEqual(d.a, 1)
        self.assertEqual(d["a"], 1)
        
        d.b = 2
        self.assertEqual(d.b, 2)
        self.assertEqual(d["b"], 2)
        
        
        del d.a
        with self.assertRaises(AttributeError):
            d.a

        del d.b
        with self.assertRaises(AttributeError):
            d.b


    def test_readwrite(self):
        d = dicy()
        d.a = 1
        d.b = 2
        d.c = dicy()
        d.c.d = 3

        s = pickle.dumps(d)
        d2 = pickle.loads(s)

        self.assertEqual(d2.a, d.a)
        self.assertEqual(d2.b, d.b)
        self.assertEqual(d2.c.d, d.c.d)
        

if __name__ == '__main__':
    unittest.main()




from dot.dicy import dicy

class ency():
    """
    'Encyclopedia' of dicts/dictos whose items can be directly dot-accessed on this object. Can have any members and
    methods, but the dictos (or dicts) that are to be dot-accessed must have their names passed in to the
    super() constructor, after being assigned to member variables with those names.
    """
    
    def __init__(self, dicto_names):  # Must be called AFTER all the dictos are initialized
        self.__dicto_names__ = dicto_names
        self.__initialized__ = True
    
    def __getattr__(self, name):
        def wrap(x):
            return x  # return dicto(x) if type(x) is dict else x # Removed: autoconversion to dicto causes a copy, preventing write access

        # print(f"Get: {name}")
        if name == "__initialized__":  # Only reached if the variable wasn't defined yet, so we know the answer
            return False
        if name in self.__dict__.keys():
            return wrap(super().__getattribute__(name))
        
        for dn in self.__dicto_names__:
            d = super().__getattribute__(dn)
            if name in d:
                return wrap(d[name])
        
        # If reaching this point, couldn't be found
        raise AttributeError("No such attribute: " + name)
    
    # Removed: autoconversion to dicto causes a copy, preventing write access
    # def __getattribute__(self, name):
    #     x = super().__getattribute__(name)
    #     if name != "__dict__" and type(x) is dict:
    #         x = dicto(x)
    #     return x
    
    def __setattr__(self, name, value):
        # print(f"Set: {name}")
        if (not self.__initialized__) or name in self.__dict__.keys():  #
            super().__setattr__(name, value)
            return
        
        for dn in self.__dicto_names__:
            d = super().__getattribute__(dn)
            if name in d:
                d[name] = value
                return
        
        # If reaching this point, couldn't be found
        super().__setattr__(name, value)
    
    def __delattr__(self, name):
        # print(f"Del: {name}")
        if name in ("__initialized__", "__dicto_names__"):
            raise AttributeError("Cannot delete attribute: " + name)
        if name in self.__dict__.keys():
            # print("deleting attribute")
            super().__delattr__(name)
            return
        
        for dn in self.__dicto_names__:
            d = super().__getattribute__(dn)
            if name in d:
                # print("deleting from dicto")
                del d[name]
                return
        
        # If reaching this point, couldn't be found
        raise AttributeError("No such attribute: " + name)
    
    def __dir__(self):
        attribs = list(self.__dict__.keys())
        for dn in self.__dicto_names__:
            d = super().__getattribute__(dn)
            attribs += list(d.keys())
        return attribs


    def __getstate__(self):
        return self.__dict__

    def __setstate__(self, state):
        self.__dict__.update(state)
        #self.__initialized__ = True


# class custom_dotcontainer_example():
#     def __init__(self, dict1: dicto = None, dict2: dict = None):
#         if dict1 is None:
#             dict1 = dicto()
#         if dict2 is None:
#             dict2 = {}
#
#         self.dict1 = dict1.copy()
#         self.dict2 = dict2.copy()
#
#         self.__initialized__ = True  # Must be the FINAL line of init
#
#     def __getattr__(self, name):
#         if name == "__initialized__":  # Only reached if the variable wasn't defined yet, so we know the answer
#             return False
#         if name in self.__dict__.keys():
#             return super().__getattribute__(name)
#         elif name in self.dict1:
#             return self.dict1[name]
#         elif name in self.dict2:
#             return self.dict2[name]
#         else:
#             raise AttributeError("No such attribute: " + name)
#
#     def __setattr__(self, name, value):
#         if not self.__initialized__ or name in self.__dict__.keys():  #
#             super().__setattr__(name, value)
#         elif self.dict1 and name in self.dict1:
#             self.dict1[name] = value
#         elif self.dict2 and name in self.dict2:
#             self.dict2[name] = value
#         else:
#             super().__setattr__(name, value)
#
#     def __delattr__(self, name):
#         if name == "__initialized__":
#             raise AttributeError("Cannot delete attribute: " + name)
#         if name in self.__dict__.keys():
#             super().__delattr__(name)
#         elif name in self.dict1:
#             del self.dict1[name]
#         elif name in self.dict2:
#             del self.dict2[name]
#         else:
#             raise AttributeError("No such attribute: " + name)
#
#     def __dir__(self):
#         return list(self.__dict__.keys()) + list(self.dict1.keys()) + list(self.dict2.keys())




class dicy(dict):
    """Dictionary which allows dot access of its items"""
    
    def __getattr__(self, name):
        if name in self:
            value = self[name]
            return value # dicto(value) if type(value) is dict else value # Removed: autoconversion to dicto causes a copy, preventing write access
        else:
            raise AttributeError("No such attribute: " + name)
    
    def __setattr__(self, name, value):
        self[name] = value
    
    def __delattr__(self, name):
        if name in self:
            del self[name]
        else:
            raise AttributeError("No such attribute: " + name)
    
    def __dir__(self):
        return list(self.__dict__.keys()) + list(self.keys())




import pickle
import unittest
from dot.dicy import dicy
from dot.ency import ency
from .helper.encyTest import encyTest, encyTestSimple

class test_ency(unittest.TestCase):
    def test(self):
        ef = encyTest()
        ef.selfTest(self)
        
            
    def test_readwriteSimple(self):

            ef = encyTestSimple()

            js = pickle.dumps(ef)
            ef2 = pickle.loads(js)

            self.assertEqual(ef2.dot1["a"], ef.dot1["a"])
            self.assertEqual(ef2.dot1["b"], ef.dot1["b"])

    def test_readwrite(self):

            ef = encyTest()

            js = pickle.dumps(ef)
            ef2 = pickle.loads(js)

            ef.selfTest(self)
            ef2.selfTest(self)


if __name__ == '__main__':
    unittest.main()



from setuptools import setup, find_packages

setup(
    name="dotsy", 
    version="0.1.0",  
    author="Levi Davis",
    author_email="levi.davis.co@gmail.com",
    description="dot-accessible dictionaries (dicy) and dictionary-containing encyclopedia classes (ency)",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/lcdavis13/dot", 
    packages=find_packages(), 
    classifiers=[
        "Programming Language :: Python :: 3", 
        "License :: OSI Approved :: MIT License", 
        "Operating System :: OS Independent", 
    ], 
    python_requires='>=3.6', 
)
